@@ -40,6 +40,7 @@
 #include "vncDesktop.h"
 #include "vncService.h"
 #include "WallpaperUtils.h"
+//#include "TsSessions.h"
 
 #if (_MSC_VER>= 1300)
 #include <fstream>
@@ -57,11 +58,9 @@ const UINT RFB_LOCAL_MOUSE = RegisterWindowMessage("WinVNC.Local.Mouse");
 
 const char szDesktopSink[] = "WinVNC desktop sink";
 
-#if 0
 // Atoms
 const char *VNC_WINDOWPOS_ATOMNAME = "VNCHooks.CopyRect.WindowPos";
-ATOM VNC_WINDOWPOS_ATOM = NULL;
-#endif
+ATOM VNC_WINDOWPOS_ATOM = (ATOM) NULL;
 
 // Static members to use with new polling algorithm
 const int vncDesktop::m_pollingOrder[32] = {
@@ -162,6 +161,12 @@ void *vncDesktopThread::run_undetached(void *arg)
 	// Save the thread's "home" desktop, under NT (no effect under 9x)
 	HDESK home_desktop = GetThreadDesktop(GetCurrentThreadId());
 
+#if 0
+	// Try to make session zero the console session
+	if (!inConsoleSession())
+		setConsoleSession();
+#endif
+
 	// Attempt to initialise and return success or failure
 	if (!m_desktop->Startup())
 	{
@@ -195,7 +200,7 @@ void *vncDesktopThread::run_undetached(void *arg)
 	MSG msg;
 	while (TRUE)
 	{
-		if (!PeekMessage(&msg, m_desktop->Window(), NULL, NULL, PM_REMOVE))
+		if (!PeekMessage(&msg, m_desktop->Window(), (UINT) NULL, (UINT) NULL, PM_REMOVE))
 		{
 			// Whenever the message queue becomes empty, we check to see whether
 			// there are updates to be passed to clients (first we make sure
@@ -403,6 +408,14 @@ vncDesktop::~vncDesktop()
 BOOL
 vncDesktop::Startup()
 {
+#if 0
+	// Currently, we just check whether we're in the console session, and
+	//   fail if not
+	if (!inConsoleSession()) {
+		vnclog.Print(LL_INTERR, VNCLOG("Console is not session zero - reconnect to restore Console session"));
+		return FALSE;
+	}
+#endif
 
 	// Configure the display for optimal VNC performance.
 	SetupDisplayForConnection();
@@ -564,7 +577,7 @@ BOOL vncDesktop::Shutdown()
 	}
 
 	// Free the WindowPos atom!
-	if (VNC_WINDOWPOS_ATOM != NULL)
+	if (VNC_WINDOWPOS_ATOM != (ATOM) NULL)
 	{
 		if (GlobalDeleteAtom(VNC_WINDOWPOS_ATOM) != 0)
 		{
@@ -684,7 +697,7 @@ vncDesktop::KillScreenSaver()
 				// Pause long enough for the screen-saver to close
 				//Sleep(2000);
 				// Reset the screen saver so it can run again
-				SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, TRUE, 0, SPIF_SENDWININICHANGE); 
+				SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, (UINT) TRUE, 0, SPIF_SENDWININICHANGE); 
 			}
 			break;
 		}
@@ -700,7 +713,6 @@ void vncDesktop::ChangeResNow()
 	}
 
 	BOOL settingsUpdated = false;
-	int i = 0;
 
 	_ASSERTE(!m_lpAlternateDevMode);
 	m_lpAlternateDevMode = new DEVMODE; // *** create an instance of DEVMODE - Jeremy Peaks
@@ -2006,7 +2018,7 @@ BOOL vncDesktop::CheckUpdates()
 		UpdateBlankScreenTimer();
 
 		// Has the display resolution or desktop changed?
-		if (m_displaychanged || !vncService::InputDesktopSelected())
+		if (m_displaychanged || !vncService::InputDesktopSelected() /*|| !inConsoleSession()*/)
 		{
 			vnclog.Print(LL_STATE, VNCLOG("display resolution or desktop changed.\n"));
 
@@ -2268,8 +2280,10 @@ inline void vncDesktop::CheckRects(vncRegion &rgn, rectlist &rects)
 #endif
 }
 
+#ifdef _MSC_VER
 // This notably improves performance when using Visual C++ 6.0 compiler
 #pragma function(memcpy, memcmp)
+#endif
 
 static const int BLOCK_SIZE = 32;
 
@@ -2693,7 +2707,8 @@ void vncDesktop::CopyRect(RECT const &rcDest, POINT ptSrc)
 
 	// Clip the destination to the screen
 	RECT rcDr2;
-	if (!IntersectRect(&rcDr2, &rcDest, &m_server->GetSharedRect()))
+	const RECT r = m_server->GetSharedRect();
+	if (!IntersectRect(&rcDr2, &rcDest, &r))
 		return;
 
 // NOTE: this is important.
@@ -2714,7 +2729,7 @@ void vncDesktop::CopyRect(RECT const &rcDest, POINT ptSrc)
 
 	// Clip the source to the screen
 	RECT rcSr2;
-	if (!IntersectRect(&rcSr2, &rcSource, &m_server->GetSharedRect()))
+	if (!IntersectRect(&rcSr2, &rcSource, &r))
 		return;
 
 	rcDr2 = MoveRect(rcSr2, mv2);
@@ -2885,7 +2900,7 @@ BOOL vncDesktop::InitVideoDriver()
 	{
 		RECT	vdesk_rect;
 		GetSourceDisplayRect(vdesk_rect);
-		BOOL b = m_videodriver->Activate(bSolicitDASD, &vdesk_rect);
+		(BOOL) m_videodriver->Activate(bSolicitDASD, &vdesk_rect);
 	}
 
 	if (!m_videodriver->CheckVersion())
@@ -3069,7 +3084,7 @@ bool	bDbgBmDump(
 		"%04u.%02u.%02u-%02u-%02u-%02u-0x%08x.bmp",
 		stm.wYear, stm.wMonth, stm.wDay,
 		stm.wHour, stm.wMinute, stm.wSecond,
-		ptr);
+		(uint) ptr);
 
 	HANDLE hFile = CreateFile(
 		szFileName,
