@@ -76,7 +76,7 @@ char genCA[] =
 "name_opt 	= ca_default		# Subject Name options\n"
 "cert_opt 	= ca_default		# Certificate field options\n"
 "\n"
-"default_days	= 365			# how long to certify for\n"
+"default_days	= 730			# how long to certify for\n"
 "default_crl_days= 30			# how long before next CRL\n"
 "default_md	= md5			# which md to use.\n"
 "preserve	= no			# keep passed DN ordering\n"
@@ -333,6 +333,13 @@ char genCert[] =
 "	echo \"Creating new x11vnc certificate and key for name: $type $name0\"\n"
 "	echo \"\"\n"
 "\n"
+"	req_args=$REQ_ARGS\n"
+"	if echo \"$req_args\" | grep 'days' > /dev/null; then\n"
+"		:\n"
+"	else\n"
+"		req_args=\"$req_args -days 730\"\n"
+"	fi\n"
+"\n"
 "	cnf=\"$DIR/tmp/cnf.$$\"\n"
 "	trap \"rm -f \\\"$cnf\\\"\" 0 1 2 15\n"
 "\n"
@@ -343,7 +350,7 @@ char genCert[] =
 "			direrror \"$DIR/CA/self.cnf.$type\"\n"
 "		fi\n"
 "		cat \"$DIR/CA/self.cnf.$type\" | sed -e \"s/%NAME/$name0/\" > \"$cnf\" || exit 1\n"
-"		\"$OPENSSL\" req -config \"$cnf\" -nodes -new -newkey rsa:2048 -x509 $REQ_ARGS \\\n"
+"		\"$OPENSSL\" req -config \"$cnf\" -nodes -new -newkey rsa:2048 -x509 $req_args \\\n"
 "			-keyout \"$DIR/$dest.key\" \\\n"
 "			-out    \"$DIR/$dest.crt\"\n"
 "	else\n"
@@ -351,7 +358,7 @@ char genCert[] =
 "			direrror \"$DIR/CA/ssl.cnf.$type\"\n"
 "		fi\n"
 "		cat \"$DIR/CA/ssl.cnf.$type\" | sed  -e \"s/%NAME/$name0/\" > \"$cnf\" || exit 1\n"
-"		\"$OPENSSL\" req -config \"$cnf\" -nodes -new -newkey rsa:2048 $REQ_ARGS \\\n"
+"		\"$OPENSSL\" req -config \"$cnf\" -nodes -new -newkey rsa:2048 $req_args \\\n"
 "			-keyout \"$DIR/$dest.key\" \\\n"
 "			-out    \"$DIR/$dest.req\"\n"
 "	fi\n"
@@ -1400,32 +1407,32 @@ char create_display[] =
 "		else\n"
 "			sxcmd=$have_xinit\n"
 "		fi\n"
-"		echo \"$sxcmd $sess -- $* -nolisten tcp -auth $authfile $FD_OPTS\" 1>&2\n"
+"		echo \"$sxcmd $sess -- $* $nolisten -auth $authfile $FD_OPTS\" 1>&2\n"
 "		if [ \"X$have_root\" != \"X\" ]; then\n"
-"			$sxcmd $sess -- $* -nolisten tcp -auth $authfile $FD_OPTS 1>&2 &\n"
+"			$sxcmd $sess -- $* $nolisten -auth $authfile $FD_OPTS 1>&2 &\n"
 "		else\n"
 "			if [ \"X$ns\" = \"X0\" ]; then\n"
-"				$have_nohup sh -c \"$sxcmd $sess -- $* -nolisten tcp -auth $authfile $FD_OPTS\" 1>&2 &\n"
+"				$have_nohup sh -c \"$sxcmd $sess -- $* $nolisten -auth $authfile $FD_OPTS\" 1>&2 &\n"
 "			else\n"
 "				# Why did we ever sleep before starting the server??\n"
-"				$have_nohup sh -c \"(sleep $ns; $sxcmd $sess -- $* -nolisten tcp -auth $authfile $FD_OPTS)\" 1>&2 &\n"
+"				$have_nohup sh -c \"(sleep $ns; $sxcmd $sess -- $* $nolisten -auth $authfile $FD_OPTS)\" 1>&2 &\n"
 "				#result=1\n"
 "			fi\n"
 "		fi\n"
 "		pid=$!\n"
 "	else\n"
 "		# need to emulate startx/xinit ourselves...\n"
-"		echo \"$* -nolisten tcp -auth $authfile $FD_OPTS\" 1>&2\n"
+"		echo \"$* $nolisten -auth $authfile $FD_OPTS\" 1>&2\n"
 "		if [ \"X$have_root\" != \"X\" ]; then\n"
-"			$have_nohup $* -nolisten tcp -auth $authfile $FD_OPTS 1>&2 &\n"
+"			$have_nohup $* $nolisten -auth $authfile $FD_OPTS 1>&2 &\n"
 "			pid=$!\n"
 "			sleep 3\n"
 "			$have_nohup $sess 1>&2 &\n"
 "		else\n"
 "			if [ \"X$ns\" = \"X0\" ]; then\n"
-"				$have_nohup sh -c \"$* -nolisten tcp -auth $authfile $FD_OPTS\" 1>&2 &\n"
+"				$have_nohup sh -c \"$* $nolisten -auth $authfile $FD_OPTS\" 1>&2 &\n"
 "			else\n"
-"				$have_nohup sh -c \"(sleep $ns; $* -nolisten tcp -auth $authfile $FD_OPTS)\" 1>&2 &\n"
+"				$have_nohup sh -c \"(sleep $ns; $* $nolisten -auth $authfile $FD_OPTS)\" 1>&2 &\n"
 "				#result=1\n"
 "			fi\n"
 "			pid=$!\n"
@@ -1719,6 +1726,8 @@ char create_display[] =
 "depth=${depth:-16}\n"
 "geom=${geom:-1280x1024}\n"
 "\n"
+"nolisten=${FD_NOLISTEN:-\"-nolisten tcp\"}\n"
+"\n"
 "if [ \"X$X11VNC_CREATE_GEOM\" != \"X\" -a \"X$FD_GEOM\" = \"X\" ]; then\n"
 "	FD_GEOM=$X11VNC_CREATE_GEOM\n"
 "fi\n"
