@@ -44,7 +44,7 @@
 #include <QtGui/QPixmap>
 #include <QtGui/QSplashScreen>
 #include <QtGui/QSplitter>
-#include <QtGui/QWorkspace>
+#include <QtGui/QPainter>
 #include <QtNetwork/QHostInfo>
 
 
@@ -56,6 +56,7 @@
 #include "italc_side_bar.h"
 #include "local_system.h"
 #include "tool_button.h"
+#include "tool_bar.h"
 #include "messagebox.h"
 
 #define DEFAULT_WINDOW_WIDTH	1005
@@ -102,8 +103,11 @@ classroomManager::classroomManager( mainWindow * _main_window,
 	// which actually is assigned after this function returns
 	_main_window->m_classroomManager = this;
 
-	m_view = new QTreeWidget( contentParent() );
-	contentParent()->layout()->addWidget( m_view );
+	QVBoxLayout * l = dynamic_cast<QVBoxLayout *>(
+						contentParent()->layout() );
+
+	m_view = new classTreeWidget( contentParent() );
+	l->addWidget( m_view );
 	m_view->setWhatsThis( tr( "This is where computers and classrooms are "
 					"managed. You can add computers or "
 					"classrooms by clicking right "
@@ -126,36 +130,33 @@ classroomManager::classroomManager( mainWindow * _main_window,
 	connect( m_view, SIGNAL( itemDoubleClicked( QTreeWidgetItem *, int ) ),
 		this, SLOT( itemDoubleClicked( QTreeWidgetItem *, int ) ) );
 	connect( m_view, SIGNAL( customContextMenuRequested( const QPoint & ) ),
-			this, SLOT( contextMenuRequest( const QPoint & ) ) );
-
+		this, SLOT( contextMenuRequest( const QPoint & ) ) );
 
 	QFont f;
 	f.setPixelSize( 12 );
 
-	m_showUserColumnCheckBox = new QCheckBox( tr( "Show usernames" ),
-			contentParent() );
-	m_showUserColumnCheckBox->setFont( f );
-	contentParent()->layout()->addWidget( m_showUserColumnCheckBox );
-	connect( m_showUserColumnCheckBox, SIGNAL( stateChanged( int ) ),
+	m_showUsernameCheckBox = new QCheckBox( tr( "Show usernames" ),
+							contentParent() );
+	m_showUsernameCheckBox->setFont( f );
+	l->addWidget( m_showUsernameCheckBox );
+	connect( m_showUsernameCheckBox, SIGNAL( stateChanged( int ) ),
 			this, SLOT( showUserColumn( int ) ) );
 
+	l->addSpacing( 5 );
 	QLabel * help_txt = new QLabel(
-		tr( "\nUse the context-menu (right mouse-button) to add/remove "
-			"computers and/or classrooms.\n\n"
-			"Once you did so you can show or hide computers by "
-			"double-clicking them.\n\nFurther actions can be "
-			"found in the context-menu of a computer or "
-			"classroom.\n" ),
-							contentParent() );
-	contentParent()->layout()->addWidget( help_txt );
+		tr( "Use the context-menu (right mouse-button) to add/remove "
+			"computers and/or classrooms." ), contentParent() );
+	l->addWidget( help_txt );
 	help_txt->setWordWrap( TRUE );
 	help_txt->setFont( f );
 
+	l->addSpacing( 16 );
+
 	m_exportToFileBtn = new QPushButton(
-			QPixmap( ":/resources/filesave.png" ),
-			tr( "Export to text-file" ),
-			contentParent() );
-	contentParent()->layout()->addWidget( m_exportToFileBtn );
+				QPixmap( ":/resources/filesave.png" ),
+						tr( "Export to text-file" ),
+							contentParent() );
+	l->addWidget( m_exportToFileBtn );
 	connect( m_exportToFileBtn, SIGNAL( clicked() ),
 			this, SLOT( clickedExportToFile() ) );
 	m_exportToFileBtn->setWhatsThis( tr( "Use this button for exporting "
@@ -170,15 +171,109 @@ classroomManager::classroomManager( mainWindow * _main_window,
 				"collect their files at the "
 				"end of the exam." ) );
 
+	setupMenus();
+
 	loadGlobalClientConfig();
 	loadPersonalConfig();
 
-	QAction * hide_teacher_clients =
-		m_quickSwitchMenu->addAction( tr( "Hide teacher computers" ) );
-	connect( hide_teacher_clients, SIGNAL( triggered( bool ) ),
+	show();
+}
+
+
+
+
+void classroomManager::setupMenus()
+{
+	QAction * act;
+	QAction * separator = new QAction( this );
+	separator->setSeparator( TRUE );
+
+	/*** quick workspace menu ***/
+
+	m_quickSwitchMenu->addAction( tr( "Hide teacher computers" ),
 					this, SLOT( hideTeacherClients() ) );
 
-	show();
+/*	m_quickSwitchMenu->addSeparator();
+
+	m_quickSwitchMenu->addAction( QPixmap( ":/resources/adjust_size.png" ),
+				tr( "Adjust windows and their size" ),
+						this, SLOT( adjustWindows() ) );
+
+	m_quickSwitchMenu->addAction(
+				QPixmap( ":/resources/auto_arrange.png" ),
+				tr( "Auto re-arrange windows" ),
+					this, SLOT( arrangeWindows() ) );*/
+
+	/*** actions for single client in context Menu ***/
+
+	m_classRoomItemActionGroup = new QActionGroup( this );
+
+	act = m_classRoomItemActionGroup->addAction( 
+			QPixmap( ":/resources/client_show.png" ),
+			tr( "Show/hide" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( showHideClient() ) );
+
+	act = m_classRoomItemActionGroup->addAction( 
+			QPixmap( ":/resources/config.png" ),
+			tr( "Edit settings" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( editClientSettings() ) );
+
+	act = m_classRoomItemActionGroup->addAction(
+			QPixmap( ":/resources/client_remove.png" ),
+			tr( "Remove" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( removeClient() ) );
+
+	m_classRoomItemActionGroup->addAction( "" )->setSeparator( TRUE );
+
+	/*** actions for single classroom in context Menu ***/
+
+	m_classRoomActionGroup = new QActionGroup( this );
+
+	act = m_classRoomActionGroup->addAction(
+			QPixmap( ":/resources/classroom_show.png" ),
+			tr( "Show all computers in classroom" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( showSelectedClassRooms() ) );
+
+	act = m_classRoomActionGroup->addAction(
+			QPixmap( ":/resources/classroom_closed.png" ),
+			tr( "Hide all computers in classroom" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( hideSelectedClassRooms() ) );
+
+	act = m_classRoomActionGroup->addAction(
+			QPixmap( ":/resources/no_user.png" ),
+			tr( "Hide teacher computers" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( hideTeacherClients() ) );
+
+	act = m_classRoomActionGroup->addAction(
+			QPixmap( ":/resources/config.png" ),
+			tr( "Edit name" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( editClassRoomName() ) );
+
+	act = m_classRoomActionGroup->addAction(
+			QPixmap( ":/resources/classroom_remove.png" ),
+			tr("Remove classroom" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( removeClassRoom() ) );
+
+	m_classRoomActionGroup->addAction( "" )->setSeparator( TRUE );
+
+	/*** common actions in context menu ***/
+
+	m_contextActionGroup = new QActionGroup( this );
+
+	act = m_contextActionGroup->addAction(
+			QPixmap( ":/resources/client_add.png" ),
+			tr( "Add computer" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( addClient() ) );
+
+	act = m_contextActionGroup->addAction(
+			QPixmap( ":/resources/classroom_add.png" ),
+			tr( "Add classroom" ) );
+	connect( act, SIGNAL( triggered() ), this, SLOT( addClassRoom() ) );
+
+	/*** Default client Menu ***/
+
+	m_clientMenu = clientMenu::createDefault( this );
+	
 }
 
 
@@ -211,8 +306,6 @@ void classroomManager::doCleanupWork( void )
 		delete m_classRoomsToRemove.first();
 		m_classRoomsToRemove.erase( m_classRoomsToRemove.begin() );
 	}
-	//savePersonalConfig();
-	//saveGlobalClientConfig();
 }
 
 
@@ -285,22 +378,39 @@ void classroomManager::savePersonalConfig( void )
 	globalsettings.setAttribute( "wincfg", QString(
 				getMainWindow()->saveState().toBase64() ) );
 
-	globalsettings.setAttribute( "demo-net-iface",
-						__demo_network_interface );
-	globalsettings.setAttribute( "demo-master-ip", __demo_master_ip );
 	globalsettings.setAttribute( "defaultdomain", __default_domain );
 	globalsettings.setAttribute( "demoquality", __demo_quality );
 	globalsettings.setAttribute( "role", __role );
 	globalsettings.setAttribute( "notooltips",
 					toolButton::toolTipsDisabled() );
+	globalsettings.setAttribute( "icononlymode",
+					toolButton::iconOnlyMode() );
 	globalsettings.setAttribute( "clientdoubleclickaction",
 						m_clientDblClickAction );
 	globalsettings.setAttribute( "showUserColumn",
-						m_showUserColumnCheckBox->isChecked() );
+						m_showUsernameCheckBox->isChecked() );
+
+	QStringList hidden_buttons;
+	foreach( QAction * a, getMainWindow()->getToolBar()->actions() )
+	{
+		if( !a->isVisible() )
+		{
+			hidden_buttons += a->text();
+		}
+	}
+	foreach( KMultiTabBarTab * tab, getMainWindow()->getSideBar()->tabs() )
+	{
+		if( !tab->isTabVisible() )
+		{
+			hidden_buttons += tab->text();
+		}
+	}
+	globalsettings.setAttribute( "toolbarcfg", hidden_buttons.join( "#" ) );
 
 	head.appendChild( globalsettings );
 
 
+
 	QDomElement root = doc.createElement( "body" );
 	italc_config.appendChild( root );
 
@@ -310,6 +420,10 @@ void classroomManager::savePersonalConfig( void )
 									FALSE );
 	}
 
+	foreach ( QDomNode node, m_customMenuConfiguration )
+	{
+		root.appendChild( node.cloneNode() );
+	}
 
 	QString xml = "<?xml version=\"1.0\"?>\n" + doc.toString( 2 );
 	if( mainWindow::ensureConfigPathExists() == FALSE )
@@ -366,10 +480,10 @@ void classroomManager::saveSettingsOfChildren( QDomDocument & _doc,
 				client_element.setAttribute( "id", c->id() );
 				if( _is_global_config )
 				{
+					client_element.setAttribute( "hostname",
+								c->hostname() );
 					client_element.setAttribute( "name",
-								c->name() );
-					client_element.setAttribute( "localip",
-								c->localIP() );
+								c->nickname() );
 					client_element.setAttribute( "mac",
 								c->mac() );
 					client_element.setAttribute( "type",
@@ -444,7 +558,7 @@ void classroomManager::getVisibleClients( QTreeWidgetItem * _p,
 			getVisibleClients( lvi, _vc );
 		}
 		else if( ( l = dynamic_cast<classRoomItem *>( lvi ) ) != NULL &&
-					l->getClient()->isVisible() )
+								l->isVisible() )
 		{
 			_vc.push_back( l->getClient() );
 		}
@@ -502,14 +616,12 @@ getMainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
 				m_winCfg = node.toElement().attribute(
 								"wincfg" );
 			}
-			// for now we only set the network-interface over which
-			// the demo should run
-			__demo_network_interface = node.toElement().
-						attribute( "demo-net-iface" );
-
-			__demo_master_ip = node.toElement().
-						attribute( "demo-master-ip" );
-
+			if( node.toElement().attribute( "toolbarcfg" ) !=
+								QString::null )
+			{
+				m_toolBarCfg = node.toElement().attribute(
+								"toolbarcfg" );
+			}
 			__demo_quality = node.toElement().
 					attribute( "demoquality" ).toInt();
 
@@ -526,9 +638,12 @@ getMainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
 			toolButton::setToolTipsDisabled(
 				node.toElement().attribute( "notooltips" ).
 								toInt() );
+			toolButton::setIconOnlyMode(
+				node.toElement().attribute( "icononlymode" ).
+								toInt() );
 			m_clientDblClickAction = node.toElement().attribute(
 					"clientdoubleclickaction" ).toInt();
-			m_showUserColumnCheckBox->setChecked(
+			m_showUsernameCheckBox->setChecked(
 				node.toElement().attribute( "showUserColumn" ).toInt() );
 			// if the attr did not exist, we got zero as value,
 			// which is not acceptable
@@ -563,8 +678,8 @@ void classroomManager::loadTree( classRoom * _parent_item,
 			if( _is_global_config )
 			{
 				// add new classroom
-				QString name = node.toElement().attribute(
-								"name" );
+				QDomElement e = node.toElement();
+				QString name = e.attribute( "name" );
 				if( _parent_item == NULL )
 				{
 					cur_item = new classRoom( name, this,
@@ -587,45 +702,41 @@ void classroomManager::loadTree( classRoom * _parent_item,
 		{
 			if( _is_global_config )
 			{
-                                client * c;
-				// add new client
-				QString local_ip = node.toElement().attribute(
-								"localip" );
-				QString mac = node.toElement().attribute(
-									"mac" );
-				QString name = node.toElement().attribute(
-								"name" );
+				QDomElement e = node.toElement();
+				QString hostname = e.hasAttribute( "hostname" )
+					? e.attribute( "hostname" )
+					: e.attribute( "localip" );
+				QString mac = e.attribute( "mac" );
+				QString nickname = e.attribute( "name" );
 
-	c = new client( local_ip, mac, name,
-				(client::types) node.toElement().
-						attribute( "type" ).toInt(),
-			_parent_item, getMainWindow(), node.toElement().
-						attribute( "id" ).toInt() );
+				// add new client
+                                client * c = new client( hostname,
+						mac,
+						nickname,
+						(client::types)e.attribute(
+							"type" ).toInt(),
+						_parent_item,
+						getMainWindow(),
+						e.attribute( "id" ).toInt() );
 				c->hide();
 			}
 			else
 			{
+				QDomElement e = node.toElement();
 				client * c = client::clientFromID(
-						node.toElement().attribute(
-							"id" ).toInt() );
+						e.attribute( "id" ).toInt() );
 				if( c == NULL )
 				{
 					continue;
 				}
-				c->move(
-			node.toElement().attribute( "x" ).toInt(),
-			node.toElement().attribute( "y" ).toInt() );
-				c->m_rasterX = node.toElement().attribute(
-								"x" ).toInt();
-				c->m_rasterY = node.toElement().attribute(
-								"y" ).toInt();
-				c->setFixedSize( node.toElement().attribute(
-								"w" ).toInt(),
-						node.toElement().attribute(
-								"h" ).toInt() );
+				c->move( e.attribute( "x" ).toInt(),
+					e.attribute( "y" ).toInt() );
+				c->m_rasterX = e.attribute( "x" ).toInt();
+				c->m_rasterY = e.attribute( "y" ).toInt();
+				c->setFixedSize( e.attribute( "w" ).toInt(),
+						e.attribute( "h" ).toInt() );
 
-				if( node.toElement().attribute( "visible" ) ==
-									"yes" )
+				if( e.attribute( "visible" ) == "yes" )
 				{
 					c->show();
 				}
@@ -635,7 +746,74 @@ void classroomManager::loadTree( classRoom * _parent_item,
 				}
 			}
 		}
-        }
+		else if( node.nodeName() == "menu" )
+		{
+			m_customMenuConfiguration.append( node.cloneNode() );
+			loadMenuElement( node.toElement() );
+		}
+	}
+}
+
+
+
+
+void classroomManager::loadMenuElement( QDomElement _e )
+{
+	if ( _e.hasAttribute( "hide" ) )
+	{
+		foreach( QAction * act, m_clientMenu->actions() )
+		{
+			if ( act->text() == _e.attribute( "hide" ) )
+			{
+				act->setVisible( FALSE );
+			}
+		}
+	}
+	else
+	{
+		QString name = _e.attribute( "remote-cmd",
+				_e.attribute( "local-cmd" ) );
+		QString icon = _e.attribute( "icon", ":resources/run.png" );
+		QString before = _e.attribute( "before" );
+
+		if ( name.isEmpty() )
+		{
+			return;
+		}
+
+		clientAction::type type = _e.hasAttribute( "remote-cmd" ) ?
+			clientAction::RemoteScript : clientAction::LocalScript;
+
+		QAction * act = new clientAction( type,
+				QIcon( icon ), name, m_clientMenu );
+
+		QString cmd;
+		for( QDomNode n = _e.firstChild(); ! n.isNull();
+			n = n.nextSibling() )
+		{
+			if ( n.isCharacterData() )
+			{
+				cmd.append( n.toCharacterData().data() );
+			}
+		}
+		act->setData( cmd );
+
+		QAction * before_act = 0;
+		if ( ! before.isEmpty() )
+		{
+			foreach( QAction * a, m_clientMenu->actions() )
+			{
+				if ( a->text() == before )
+				{
+					before_act = a;
+					break;
+				}
+			}
+		}
+
+		/* equal to addAction if before_act = 0 */
+		m_clientMenu->insertAction( before_act, act );
+	}
 }
 
 
@@ -673,7 +851,9 @@ void classroomManager::loadGlobalClientConfig( void )
 		return;
 	}
 
-	if( !m_domTree.setContent( &cfg_file ) )
+	QDomDocument domTree;
+
+	if( !domTree.setContent( &cfg_file ) )
 	{
 		if( splashScreen != NULL )
 		{
@@ -695,7 +875,7 @@ void classroomManager::loadGlobalClientConfig( void )
 
 
 	// get the head information from the DOM
-	QDomElement root = m_domTree.documentElement();
+	QDomElement root = domTree.documentElement();
 	QDomNode node = root.firstChild();
 
 	// create the tree view out of the DOM
@@ -740,7 +920,9 @@ void classroomManager::loadPersonalConfig( void )
 		return;
 	}
 
-	if( !m_domTree.setContent( &cfg_file ) )
+	QDomDocument domTree;
+
+	if( !domTree.setContent( &cfg_file ) )
 	{
 		if( splashScreen != NULL )
 		{
@@ -761,7 +943,7 @@ void classroomManager::loadPersonalConfig( void )
 
 
 	// get the head information from the DOM
-	QDomElement root = m_domTree.documentElement();
+	QDomElement root = domTree.documentElement();
 	QDomNode node = root.firstChild();
 
 	while( !node.isNull() )
@@ -790,45 +972,6 @@ void classroomManager::loadPersonalConfig( void )
 
 
 
-void classroomManager::createActionMenu( QMenu * _m )
-{
-/*for( int i = 0; i < m_view->topLevelItemCount(); ++i )
-	{
-		QTreeWidgetItem * lvi = m_view->topLevelItem( i );
-		if( lvi->childCount() )
-		{
-			createActionMenu( lvi, _m );
-		}
-	}*/
-}
-
-
-
-
-void classroomManager::createActionMenu( QTreeWidgetItem * _p, QMenu * _m )
-{
-/*	QMenu * root = _m->addMenu( _p->text( 0 ) );
-
-	classRoomItem * l = NULL;
-
-	for( int i = 0; i < _p->childCount(); ++i )
-	{
-		QTreeWidgetItem * lvi = _p->child( i );
-		if( lvi->childCount() )
-		{
-			createActionMenu( lvi, root );
-		}
-		else if( ( l = dynamic_cast<classRoomItem *>( lvi ) ) != NULL )
-		{
-			l->getClient()->createActionMenu(
-				root->addMenu( l->getClient()->fullName() ) );
-		}
-	}*/
-}
-
-
-
-
 void classroomManager::updateClients( void )
 {
 	QVector<client *> vc = visibleClients();
@@ -846,9 +989,6 @@ void classroomManager::updateClients( void )
 
 
 
-
-
-
 void classroomManager::clickedExportToFile( void )
 {
 	QString outfn = QFileDialog::getSaveFileName( this,
@@ -865,7 +1005,7 @@ void classroomManager::clickedExportToFile( void )
 	QVector<client *> clients = getLoggedInClients();
 	foreach( client * cl, clients) 
 	{
-		output += cl->user() + "\t@ " + cl->name() + " [" + cl->localIP() + "]\n";
+		output += cl->user() + "\t@ " + cl->name() + " [" + cl->hostname() + "]\n";
 	}
 
 	QFile outfile( outfn );
@@ -877,20 +1017,6 @@ void classroomManager::clickedExportToFile( void )
 
 
 
-inline void classroomManager::cmdToVisibleClients( client::clientCmds _cmd,
-						const QString & _u_data )
-{
-	QVector<client *> vc = visibleClients();
-	foreach( client * cl, vc )
-	{
-		cl->processCmd( _cmd, _u_data );
-	}
-}
-
-
-
-
-
 void classroomManager::changeGlobalClientMode( int _mode )
 {
 	client::modes new_mode = static_cast<client::modes>( _mode );
@@ -898,12 +1024,11 @@ void classroomManager::changeGlobalClientMode( int _mode )
 					new_mode == client::Mode_Overview )
 	{
 		m_globalClientMode = new_mode;
-		isdConnection * conn = getMainWindow()->localISD();
 		QVector<client *> vc = visibleClients();
 
 		foreach( client * cl, vc )
 		{
-			cl->changeMode( m_globalClientMode, conn );
+			cl->changeMode( m_globalClientMode );
 		}
 	}
 }
@@ -913,9 +1038,8 @@ void classroomManager::changeGlobalClientMode( int _mode )
 
 void classroomManager::powerOnClients( void )
 {
-/*	progressInformation pi(
-		tr( "Please wait, while the clients are being turned on." ) );*/
-	cmdToVisibleClients( client::Cmd_PowerOn );
+	clientAction action( clientAction::PowerOn, this );
+	action.process( visibleClients(), clientAction::VisibleClients );
 }
 
 
@@ -923,14 +1047,8 @@ void classroomManager::powerOnClients( void )
 
 void classroomManager::remoteLogon( void )
 {
-	remoteLogonDialog mld( this );
-	if( mld.exec() == QDialog::Accepted &&
-		!mld.userName().isEmpty() && !mld.password().isEmpty() )
-	{
-		cmdToVisibleClients( client::Cmd_LogonUser,
-				mld.userName() + "*" + mld.password() +
-							"*" + mld.domain() );
-	}
+	clientAction action( clientAction::LogonUser, this );
+	action.process( visibleClients(), clientAction::VisibleClients );
 }
 
 
@@ -938,44 +1056,19 @@ void classroomManager::remoteLogon( void )
 
 void classroomManager::powerDownClients( void )
 {
-	if( QMessageBox::question( window(), tr( "Power down computers" ),
-					tr( "Are you sure want to power down "
-						"all visible computers?" ),
-					QMessageBox::Yes, QMessageBox::No ) ==
-							QMessageBox::Yes )
-	{
-		/*progressInformation pi( tr( "Please wait, while the computers "
-						"are being powered down." ) );*/
-		cmdToVisibleClients( client::Cmd_PowerDown );
-	}
+	clientAction action( clientAction::PowerDown, this );
+	action.process( visibleClients(), clientAction::VisibleClients );
 }
 
 
 
 
-void classroomManager::logoutUser( void )
+void classroomManager::directSupport( void )
 {
-	if( QMessageBox::question( window(), tr( "Logout user" ),
-			tr( "Are you sure want logout all users on all "
-							"visible computers ?" ),
-					QMessageBox::Yes, QMessageBox::No ) ==
-							QMessageBox::Yes )
+	const QString h = supportDialog::getHost( this );
+	if( !h.isEmpty() )
 	{
-		cmdToVisibleClients( client::Cmd_LogoutUser );
-	}
-}
-
-
-
-
-void classroomManager::execCmds( void )
-{
-	QString cmds;
-
-	cmdInputDialog cmd_input_dialog( cmds, this );
-	if( cmd_input_dialog.exec() == QDialog::Accepted && cmds != "" )
-	{
-		cmdToVisibleClients( client::Cmd_ExecCmds, cmds );
+		getMainWindow()->remoteControlDisplay( h );
 	}
 }
 
@@ -984,15 +1077,8 @@ void classroomManager::execCmds( void )
 
 void classroomManager::sendMessage( void )
 {
-	QString msg;
-
-	textMessageDialog tmd( msg, this );
-	if( tmd.exec() == QDialog::Accepted && msg != "" )
-	{
-		/*progressInformation pi(
-			tr( "Please wait, while the message is being sent." ) );*/
-		cmdToVisibleClients( client::Cmd_SendTextMessage, msg );
-	}
+	clientAction action( clientAction::SendTextMessage, this );
+	action.process( visibleClients(), clientAction::VisibleClients );
 }
 
 
@@ -1078,8 +1164,8 @@ void classroomManager::adjustWindows( void )
 		foreach( client * cl, vc )
 		{
 			cl->setFixedSize( nw - decor_w, nh - decor_h );
-			cl->move( static_cast<int>( cl->m_rasterX * nw ),
-				static_cast<int>( cl->m_rasterY * nh ) );
+			cl->move( static_cast<int>( cl->m_rasterX * nw )+1,
+				static_cast<int>( cl->m_rasterY * nh )+1 );
 		}
 		getMainWindow()->workspace()->updateGeometry();
 	}
@@ -1088,14 +1174,6 @@ void classroomManager::adjustWindows( void )
 
 
 
-bool clientIPSort( const client * _1, const client * _2 )
-{
-	return( _1->localIP() < _2->localIP() );
-}
-
-
-
-
 void classroomManager::arrangeWindows( void )
 {
 	QVector<client *> vc = visibleClients();
@@ -1113,8 +1191,6 @@ void classroomManager::arrangeWindows( void )
 		const int ww = avail_w / win_per_line;
 		const int wh = avail_h / win_per_row;
 
-		qSort( vc.begin(), vc.end(), clientIPSort );
-
 		int i = 0;
 		foreach( client * cl, vc )
 		{
@@ -1271,104 +1347,96 @@ void classroomManager::itemDoubleClicked( QTreeWidgetItem * _i, int )
 
 void classroomManager::contextMenuRequest( const QPoint & _pos )
 {
-	QMenu * context_menu = new QMenu( this );
-
 	QTreeWidgetItem * i = m_view->itemAt( _pos );
-	m_view->setCurrentItem( i );
 	classRoomItem * cri = dynamic_cast<classRoomItem *>( i );
 	classRoom * cr = dynamic_cast<classRoom *>( i );
 
-	if( cri != NULL )
+	QMenu * contextMenu = new QMenu( this );
+	QMenu * subMenu;
+
+	if ( m_view->selectedItems().size() > 1 )
 	{
-		QMenu * single_clients_submenu = context_menu->addMenu(
-					QPixmap( ":/resources/client.png" ),
-							tr( "Actions" ) );
-
-		cri->getClient()->createActionMenu( single_clients_submenu );
-
-		context_menu->addSeparator();
-
-		context_menu->addAction(
-				QPixmap( ":/resources/client_show.png" ),
-						tr( "Show/hide" ),
-					this, SLOT( showHideClient() ) );
-
-		context_menu->addAction(
-				QPixmap( ":/resources/config.png" ),
-						tr( "Edit settings" ), this,
-					SLOT( editClientSettings() ) );
-
-		context_menu->addAction(
-				QPixmap( ":/resources/client_remove.png" ),
-						tr( "Remove" ), this,
-						SLOT( removeClient() ) );
-
-		context_menu->addSeparator();
-
+		/* multiselection */
+		subMenu = new clientMenu( tr( "Actions for selected" ),
+			m_clientMenu->actions(), contextMenu );
+		connect( subMenu, SIGNAL( triggered( QAction * ) ),
+			this, SLOT( clientMenuTriggered( QAction * ) ) );
+		contextMenu->addMenu( subMenu );
+		contextMenu->addSeparator();
+	}
+	else if( cri != NULL )
+	{
+		/* single client */
+		subMenu = new clientMenu( tr( "Actions" ),
+			m_clientMenu->actions(), contextMenu, clientMenu::FullMenu );
+		connect( subMenu, SIGNAL( triggered( QAction * ) ),
+			this, SLOT( clientMenuTriggered( QAction * ) ) );
+		contextMenu->addMenu( subMenu );
+		contextMenu->addSeparator();
+		contextMenu->addActions( m_classRoomItemActionGroup->actions() );
 	}
 	else if( cr != NULL )
 	{
-		QMenu * actions_for_classroom_submenu = context_menu->addMenu(
-				QPixmap( ":/resources/classroom_opened.png" ),
-				tr( "Actions for %1" ).arg( cr->text( 0 ) ) );
-		cr->createActionMenu( actions_for_classroom_submenu, FALSE );
-
-		context_menu->addSeparator();
-
-		context_menu->addAction(
-				QPixmap( ":/resources/classroom_show.png" ),
-				tr( "Show all computers in classroom" ),
-				this, SLOT( showSelectedClassRooms() ) );
-
-		context_menu->addAction(
-				QPixmap( ":/resources/classroom_closed.png" ),
-				tr( "Hide all computers in classroom" ),
-				this, SLOT( hideSelectedClassRooms() ) );
-
-		context_menu->addAction(
-				QPixmap( ":/resources/no_user.png" ),
-				tr( "Hide teacher computers" ),
-				this, SLOT( hideTeacherClients() ) );
-
-		context_menu->addAction(
-				QPixmap( ":/resources/config.png" ),
-				tr( "Edit name" ),
-				this, SLOT( editClassRoomName() ) );
-
-		context_menu->addAction(
-				QPixmap( ":/resources/classroom_remove.png" ),
-				tr("Remove classroom" ),
-				this, SLOT( removeClassRoom() ) );
-
-		context_menu->addSeparator();
-
+		/* single classroom */
+		subMenu = new clientMenu( tr( "Actions for %1" ).arg( cr->text(0) ),
+			m_clientMenu->actions(), contextMenu );
+		connect( subMenu, SIGNAL( triggered( QAction * ) ),
+			cr, SLOT( clientMenuTriggered( QAction * ) ) );
+		contextMenu->addMenu( subMenu );
+		contextMenu->addSeparator();
+		contextMenu->addActions( m_classRoomActionGroup->actions() );
 	}
-	else if( m_classRooms.size() )
+	else
 	{ 
-		QMenu * actions_for_classroom_submenu = context_menu->addMenu(
-				QPixmap( ":/resources/classroom_closed.png" ),
-				tr( "Action for whole classroom" ) );
-		for( QVector<classRoom *>::iterator it = m_classRooms.begin();
-						it != m_classRooms.end(); ++it )
+		/* no items */
+		foreach ( classRoom * cr, m_classRooms )
 		{
-			( *it )->createActionMenu(
-						actions_for_classroom_submenu );
+			subMenu = new clientMenu( tr( "Actions for %1" ).arg( cr->text(0) ),
+				m_clientMenu->actions(), contextMenu );
+			connect( subMenu, SIGNAL( triggered( QAction * ) ),
+				cr, SLOT( clientMenuTriggered( QAction * ) ) );
+			contextMenu->addMenu( subMenu );
 		}
 
-		context_menu->addSeparator();
+		contextMenu->addSeparator();
 	}
 
-	context_menu->addAction( QPixmap( ":/resources/client_add.png" ),
-					tr( "Add computer" ),
-					this, SLOT( addClient() ) );
+	contextMenu->addActions( m_contextActionGroup->actions() );
 
-	context_menu->addAction( QPixmap( ":/resources/classroom_add.png" ),
-					tr( "Add classroom" ),
-					this, SLOT( addClassRoom() ) );
+	contextMenu->exec( QCursor::pos() );
 
-	context_menu->exec( QCursor::pos() );
+	delete contextMenu;
+}
 
-	delete context_menu;
+
+
+void classroomManager::clientMenuRequest()
+{
+	bool fullMenu = ( selectedItems().size() == 1 );
+
+	QMenu * menu = new clientMenu( tr( "Actions" ), m_clientMenu->actions(),
+					this, fullMenu );
+	connect( menu, SIGNAL( triggered( QAction * ) ),
+		this, SLOT( clientMenuTriggered( QAction * ) ) );
+
+	menu->exec( QCursor::pos() );
+
+	delete menu;
+}
+
+
+
+
+void classroomManager::clientMenuTriggered( QAction * _action )
+{
+	QVector<client *> clients;
+
+	foreach ( classRoomItem * cri, selectedItems() )
+	{
+		clients.append( cri->getClient() );
+	}
+
+	clientAction::process( _action, clients );
 }
 
 
@@ -1383,6 +1451,14 @@ QVector<classRoomItem *> classroomManager::selectedItems( void )
 		getSelectedItems( m_view->topLevelItem( i ), vc );
 	}
 
+	/* Move the currentItem to the beginning of the list */
+	classRoomItem * current = dynamic_cast<classRoomItem *>( m_view->currentItem() );
+	if ( vc.contains( current ) )
+	{
+		vc.remove( vc.indexOf( current ));
+		vc.push_front( current );
+	}
+
 	return( vc );
 }
 
@@ -1391,23 +1467,20 @@ QVector<classRoomItem *> classroomManager::selectedItems( void )
 
 void classroomManager::getSelectedItems( QTreeWidgetItem * _p,
 					QVector<classRoomItem *> & _vc,
-								bool _add_all )
+					bool _add_all )
 {
-	for( int i = 0; i < _p->childCount(); ++i )
+	bool select = _add_all || _p->isSelected();
+	if( _p->childCount() )
 	{
-		QTreeWidgetItem * lvi = _p->child( i );
-		if( lvi->childCount() )
+		for( int i = 0; i < _p->childCount(); ++i )
 		{
-			getSelectedItems( lvi, _vc,
-						m_view->isItemSelected( lvi ) );
-		}
-		else if( _add_all ||
-			( dynamic_cast<classRoomItem *>( lvi ) != NULL &&
-					m_view->isItemSelected( lvi ) ) )
-		{
-			_vc.push_back( dynamic_cast<classRoomItem *>( lvi ) );
+			getSelectedItems( _p->child( i ), _vc, select );
 		}
 	}
+	else if( dynamic_cast<classRoomItem *>( _p ) && select )
+	{
+		_vc.push_back( dynamic_cast<classRoomItem *>( _p ) );
+	}
 }
 
 
@@ -1423,7 +1496,7 @@ void classroomManager::showHideClient( void )
 		bool all_shown = TRUE;
 		foreach( classRoomItem * cri, si )
 		{
-			if( cri->getClient()->isVisible() == FALSE )
+			if( cri->getClient()->isVisible() )
 			{
 				all_shown = FALSE;
 				break;
@@ -1602,17 +1675,34 @@ void classroomManager::removeClassRoom( void )
 			continue;
 		}
 
+		removeClassRoom( cr );
+	}
+}
+
+
+
+
+void classroomManager::removeClassRoom( classRoom * cr )
+{
 		m_view->setItemHidden( cr, TRUE );
 
-		QVector<client *> vc;
-		getVisibleClients( cr, vc );
-		foreach( client * cl, vc )
+		for ( int i = 0 ; i < cr->childCount(); ++i )
 		{
-			cl->hide();
-			m_clientsToRemove.push_back( cl );
+			if ( classRoomItem * cri =
+					dynamic_cast<classRoomItem *>( cr->child( i ) ) )
+			{
+				client * cl = cri->getClient();
+				cl->hide();
+				m_clientsToRemove.push_back( cl );
+			}
+			else if ( classRoom * childCr =
+					dynamic_cast<classRoom *>( cr->child( i ) ) )
+			{
+				removeClassRoom( childCr );
+			}
 		}
+
 		m_classRoomsToRemove.push_back( cr );
-	}
 }
 
 
@@ -1676,14 +1766,22 @@ void classroomManager::addClassRoom( void )
 	if( ok && !classroom_name.isEmpty() )
 	{
 		classRoom * sel_cr = NULL;
-		foreach( classRoom * cr, m_classRooms )
+		foreach( QTreeWidgetItem * item, m_view->selectedItems() )
 		{
-			if( m_view->isItemSelected( cr ) )
+			sel_cr = dynamic_cast<classRoom *>( item );
+
+			if ( !sel_cr && dynamic_cast<classRoomItem *>( item ) )
+			{
+				sel_cr = dynamic_cast<classRoom *>( 
+					item->parent() );
+			}
+
+			if ( sel_cr )
 			{
-				sel_cr = cr;
 				break;
 			}
 		}
+
 		if( sel_cr != NULL )
 		{
 			m_classRooms.push_back( new classRoom( classroom_name,
@@ -1726,6 +1824,229 @@ void classroomManager::showUserColumn( int _show )
 
 
 
+
+
+
+
+classTreeWidget::classTreeWidget( QWidget * _parent ) :
+	QTreeWidget( _parent ),
+	m_clientPressed( NULL )
+{
+	setDragEnabled( true );
+	setAcceptDrops( true );
+	setDropIndicatorShown( true ); 
+	setDragDropMode( QAbstractItemView::InternalMove );
+
+	connect( this, SIGNAL( itemSelectionChanged( void ) ),
+		this, SLOT( itemSelectionChanged( void ) ) );
+}
+
+
+
+
+void classTreeWidget::mousePressEvent( QMouseEvent * _me )
+{
+	classRoomItem * item = dynamic_cast<classRoomItem *>( itemAt( _me->pos() ) );
+
+	if( item && _me->button() == Qt::LeftButton )
+	{
+		m_clientPressed = item->getClient();
+		m_clientPressed->zoom();
+	}
+
+	if( item && _me->button() == Qt::RightButton )
+	{
+		if ( ! item->isSelected() )
+		{
+			clearSelection();
+			item->setSelected( TRUE );
+		}
+	}
+
+	QTreeWidget::mousePressEvent( _me );
+}
+
+
+
+
+void classTreeWidget::mouseMoveEvent( QMouseEvent * _me )
+{
+	if ( m_clientPressed ) 
+	{
+		m_clientPressed->zoomBack();
+		m_clientPressed = NULL;
+	}
+
+	QTreeWidget::mouseMoveEvent( _me );
+}
+
+
+
+
+void classTreeWidget::mouseReleaseEvent( QMouseEvent * _me )
+{
+	if ( m_clientPressed ) 
+	{
+		m_clientPressed->zoomBack();
+		m_clientPressed = NULL;
+	}
+
+	QTreeWidget::mouseReleaseEvent( _me );
+}
+
+
+
+
+bool classTreeWidget::droppingOnItself( QTreeWidgetItem * _target )
+{
+    QList<QTreeWidgetItem *> selected = selectedItems();
+    while ( _target )
+    {
+	    if ( selected.contains( _target ) )
+		    return true;
+	    _target = dynamic_cast<QTreeWidgetItem * >( _target->parent() );
+    }
+    return false;
+}
+
+
+
+
+void classTreeWidget::dragMoveEvent( QDragMoveEvent * _e )
+{
+	if ( _e->source() == this ) 
+	{
+		int clients_selected = 0;
+		foreach( QTreeWidgetItem * item, selectedItems() )
+		{
+			if ( dynamic_cast<classRoomItem *>( item ) )
+			{
+				clients_selected++;
+			}
+		}
+
+		QTreeWidgetItem * target = itemAt( _e->pos() );
+		
+		/* Don't drop clients to the root nor
+		 * classroom to its own child */
+		if ( ( clients_selected && ! target ) ||
+			droppingOnItself( target ) )
+		{
+			_e->ignore();
+		}
+		else
+		{
+			_e->setDropAction( Qt::MoveAction );
+			_e->accept();
+		}
+	}
+}
+
+
+
+
+void classTreeWidget::dropEvent( QDropEvent * _e )
+{
+    if ( _e->source() == this &&
+    	dragDropMode() == QAbstractItemView::InternalMove )
+    {
+		QTreeWidgetItem * target = itemAt( _e->pos() );
+
+		/* Use client's parent as target */
+		if ( dynamic_cast<classRoomItem *>( target ))
+		{
+			target = target->parent();
+		}
+
+		/* Workaround for Qt bug #155700 (fixed in Qt 4.3.4) */
+		bool sortingEnabled = isSortingEnabled();
+		setSortingEnabled( false );
+
+		/* Move selected items */
+		foreach ( QTreeWidgetItem * item, selectedItems() )
+		{
+			if ( item != target )
+			{
+				QTreeWidgetItem * parent = item->parent();
+				if ( parent )
+				{
+					parent->takeChild( parent->indexOfChild( item ) );
+				}
+				else
+				{
+					takeTopLevelItem( indexOfTopLevelItem( item ) );
+				}
+
+				if ( target ) 
+				{
+					target->addChild( item );
+				}
+				else
+				{
+					addTopLevelItem( item );
+				}
+			}
+		}
+
+		setSortingEnabled( sortingEnabled );
+
+		_e->accept();
+    }
+}
+
+
+
+
+/* Update client windows quickly after selections have changed */
+void classTreeWidget::itemSelectionChanged( void )
+{
+
+	/* update old selections */
+	foreach( QTreeWidgetItem * item, m_selectedItems )
+	{
+		classRoomItem * cri = dynamic_cast<classRoomItem *>( item );
+		if ( cri )
+		{
+			cri->getClient()->update();
+		}
+	}
+
+	m_selectedItems = selectedItems();
+
+	/* update new selections - some clients may belong to both
+	 * lists and are updated here twice. */
+	foreach( QTreeWidgetItem * item, m_selectedItems )
+	{
+		classRoomItem * cri = dynamic_cast<classRoomItem *>( item );
+		if ( cri )
+		{
+			cri->getClient()->update();
+		}
+	}
+}
+
+
+
+
+/* Set to current but don't select it automatically */
+void classTreeWidget::setCurrentItem( QTreeWidgetItem * _item )
+{
+	QModelIndex index = indexFromItem( _item );
+
+	if ( index.isValid() )
+	{
+		selectionModel()->setCurrentIndex( index,
+				QItemSelectionModel::NoUpdate );
+	}
+}
+
+
+
+
+
+
+
+
 classRoom::classRoom( const QString & _name,
 					classroomManager * _classroom_manager,
 						QTreeWidgetItem * _parent ) :
@@ -1760,156 +2081,17 @@ classRoom::~classRoom()
 
 
 
-void classRoom::createActionMenu( QMenu * _m, bool _add_sub_menu )
+void classRoom::clientMenuTriggered( QAction * _action )
 {
-	QMenu * this_classroom_submenu = _m;
-	QMenu * orig_root = _m;
+	QVector<client *> clients;
+	classroomManager::getVisibleClients( this, clients );
 
-	if( _add_sub_menu )
-	{
-		orig_root = this_classroom_submenu;
-		this_classroom_submenu = _m->addMenu( text( 0 ) );
-	}
-	connect( this_classroom_submenu, SIGNAL( triggered( QAction * ) ),
-			this, SLOT( processCmdOnAllClients( QAction * ) ) );
-	for( int i = client::Cmd_SendTextMessage; i < client::Cmd_CmdCount;
-									++i )
-	{
-		if( i == client::Cmd_ExecCmds )
-		{
-			this_classroom_submenu =
-				this_classroom_submenu->addMenu( QPixmap(
-					":/resources/config.png" ),
-						tr( "Administration" ) );
-
-			connect( this_classroom_submenu,
-					SIGNAL( triggered( QAction * ) ),
-				this,
-				SLOT( processCmdOnAllClients( QAction * ) ) );
-		}
-		QAction * cur = this_classroom_submenu->addAction(
-				QPixmap( QString( ":/resources/" ) +
-						client::s_commands[i].m_icon ),
-				client::tr( client::s_commands[i].m_name ) );
-		cur->setData( i );
-
-		if( client::s_commands[i].m_insertSep )
-		{
-			this_classroom_submenu->addSeparator();
-		}
-	}
+	clientAction::process( _action, clients );
 }
 
 
 
 
-void classRoom::processCmdOnAllClients( QAction * _action )
-{
-	QVector<client *> vc;
-
-	// get all visible clients for this classroom
-	classroomManager::getVisibleClients( this, vc );
-
-	QString u_data = CONFIRM_NO;
-
-	int cmd = _action->data().toInt();
-
-	// may be we have to do something (e.g. confirm) before executing cmd
-	switch( static_cast<client::clientCmds>( cmd ) )
-	{
-		case client::Cmd_SendTextMessage:
-		{
-			textMessageDialog tmd( u_data, m_classroomManager );
-			if( tmd.exec() != QDialog::Accepted || u_data == "" )
-			{
-				return;
-			}
-			break;
-		}
-
-		case client::Cmd_LogonUser:
-		{
-			remoteLogonDialog mld( m_classroomManager );
-			if( mld.exec() == QDialog::Accepted &&
-				!mld.userName().isEmpty() &&
-					!mld.password().isEmpty() )
-			{
-				u_data = mld.userName() + "*" + mld.password() +
-							"*" + mld.domain();
-				break;
-			}
-			return;
-		}
-
-		case client::Cmd_LogoutUser:
-		{
-			if( QMessageBox::question( m_classroomManager->window(),
-					classroomManager::tr( "Logout user" ),
-				classroomManager::tr( "Are you sure want logout "
-							"all users on all "
-							"visible computers ?" ),
-						QMessageBox::Yes,
-						QMessageBox::No ) ==
-							QMessageBox::No )
-			{
-				return;
-			}
-			break;
-		}
-
-		case client::Cmd_ExecCmds:
-		{
-			cmdInputDialog cmd_input_dialog( u_data, NULL );
-			if( cmd_input_dialog.exec() != QDialog::Accepted ||
-								u_data == "" )
-			{
-				return;
-			}
-			break;
-		}
-
-		case client::Cmd_Reboot:
-		{
-			if( QMessageBox::question( m_classroomManager->window(),
-				classroomManager::tr( "Reboot computers" ),
-				classroomManager::tr( "Are you sure want to "
-					"reboot all visible computers?" ),
-					QMessageBox::Yes, QMessageBox::No ) ==
-							QMessageBox::No )
-			{
-				return;
-			}
-			break;
-		}
-		case client::Cmd_PowerDown:
-		{
-			if( QMessageBox::question( m_classroomManager->window(),
-				classroomManager::tr( "Power down computers" ),
-				classroomManager::tr( "Are you sure want to "
-						"power down all visible "
-								"computers?" ),
-					QMessageBox::Yes, QMessageBox::No ) ==
-							QMessageBox::No )
-			{
-				return;
-			}
-			break;
-		}
-		default:
-			break;
-	}
-
-	for( QVector<client *>::iterator it = vc.begin(); it != vc.end(); ++it )
-	{
-		( *it )->processCmd( static_cast<client::clientCmds>( cmd ),
-								u_data );
-	}
-}
-
-
-
-
-
 void classRoom::switchToClassRoom( void )
 {
 	m_classroomManager->hideAllClassRooms();
@@ -1927,6 +2109,7 @@ void classRoom::switchToClassRoom( void )
 
 classRoomItem::classRoomItem( client * _client, QTreeWidgetItem * _parent ) :
 	QTreeWidgetItem( _parent, QStringList( _client->name() ) ),
+	m_visible( FALSE ),
 	m_client( _client )
 {
 	if( s_clientPixmap == NULL )
@@ -1940,8 +2123,11 @@ classRoomItem::classRoomItem( client * _client, QTreeWidgetItem * _parent ) :
 					":/resources/client_visible.png" );
 	}
 
+	setFlags( Qt::ItemIsSelectable | Qt::ItemIsDragEnabled |
+		Qt::ItemIsEnabled );
+
 	setVisible( m_client->isVisible() );
-	setText( 1, m_client->localIP() );
+	setText( 1, m_client->hostname() );
 	setUser( m_client->user() );
 }
 
@@ -1958,6 +2144,7 @@ classRoomItem::~classRoomItem()
 
 void classRoomItem::setVisible( const bool _obs )
 {
+	m_visible = _obs;
 	if( _obs == FALSE )
 	{
 		setIcon( 0, *s_clientPixmap );
