@@ -1,7 +1,7 @@
 /*
  * config_widget.cpp - implementation of configuration-widget for side-bar
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -29,7 +29,6 @@
 #include <QtGui/QComboBox>
 #include <QtGui/QMessageBox>
 #include <QtNetwork/QHostInfo>
-#include <QtNetwork/QNetworkInterface>
 
 
 #include "config_widget.h"
@@ -58,45 +57,6 @@ configWidget::configWidget( mainWindow * _main_window, QWidget * _parent ) :
 							updateIntervalSB );
 
 
-	QList<QNetworkInterface> ifs = QNetworkInterface::allInterfaces();
-	for( QList<QNetworkInterface>::const_iterator it = ifs.begin();
-							it != ifs.end(); ++it )
-	{
-		QList<QNetworkAddressEntry> nae = it->addressEntries();
-		for( QList<QNetworkAddressEntry>::const_iterator it2 =
-								nae.begin();
-						it2 != nae.end(); ++it2 )
-		{
-			const QString txt = it->name() + ": " +
-						it2->ip().toString();
-			interfaceCB->addItem( txt );
-			if( __demo_network_interface == it->name() ||
-				( __demo_network_interface.isEmpty() &&
-							it->name() != "lo" ) )
-			{
-				if( nae.size() > 1 &&
-					!__demo_master_ip.isEmpty() &&
-					__demo_master_ip != it2->ip().toString() )
-				{
-					continue;
-				}
-				interfaceCB->setCurrentIndex(
-						interfaceCB->findText( txt ) );
-				__demo_network_interface = it->name();
-				__demo_master_ip = it2->ip().toString();
-			}
-		}
-	}
-
-	// as fallback use host-name
-	if( __demo_master_ip.isEmpty() )
-	{
-		__demo_master_ip = QHostInfo::localHostName();
-	}
-
-	connect( interfaceCB, SIGNAL( activated( const QString & ) ), this,
-				SLOT( interfaceSelected( const QString & ) ) );
-
 	demoQualityCB->setCurrentIndex( __demo_quality );
 
 	connect( demoQualityCB, SIGNAL( activated( int ) ), this,
@@ -113,6 +73,10 @@ configWidget::configWidget( mainWindow * _main_window, QWidget * _parent ) :
 	connect( balloonToolTips, SIGNAL( toggled( bool ) ),
 			this, SLOT( toggleToolButtonTips( bool ) ) );
 
+	iconOnlyToolButtons->setChecked( toolButton::iconOnlyMode() );
+	connect( iconOnlyToolButtons, SIGNAL( toggled( bool ) ),
+			this, SLOT( toggleIconOnlyToolButtons( bool ) ) );
+
 	domainEdit->setText( __default_domain );
 	connect( domainEdit, SIGNAL( textChanged( const QString & ) ),
 			this, SLOT( domainChanged( const QString & ) ) );
@@ -134,26 +98,6 @@ configWidget::~configWidget()
 
 
 
-void configWidget::interfaceSelected( const QString & _if_name )
-{
-	if( _if_name.section( ' ', -1 ) == "127.0.0.1" )
-	{
-		QMessageBox::warning( NULL,
-			tr( "Warning" ),
-			tr( "You are trying to use the local loopback-device "
-				"as network-interface. This will never work, "
-				"because the local loopback is just the last "
-				"alternative for running iTALC if no other "
-				"network-interface was found." ),
-				QMessageBox::Ok, QMessageBox::NoButton );
-	}
-	__demo_network_interface = _if_name.section( ':', 0, 0 );
-	__demo_master_ip = _if_name.section( ' ', -1 );
-}
-
-
-
-
 void configWidget::demoQualitySelected( int _q )
 {
 	__demo_quality = _q;
@@ -178,6 +122,14 @@ void configWidget::toggleToolButtonTips( bool _on )
 
 
 
+void configWidget::toggleIconOnlyToolButtons( bool _on )
+{
+	toolButton::setIconOnlyMode( _on );
+}
+
+
+
+
 void configWidget::domainChanged( const QString & _domain )
 {
 	__default_domain = _domain;
