@@ -52,7 +52,6 @@
 #include "overview_widget.h"
 #include "snapshot_list.h"
 #include "config_widget.h"
-#include "support_widget.h"
 #include "messagebox.h"
 #include "tool_button.h"
 #include "tool_bar.h"
@@ -61,7 +60,11 @@
 #include "remote_control_widget.h"
 
 
+QSystemTrayIcon * __systray_icon = NULL;
+
+
 extern int __isd_port;
+extern QString __isd_host;
 
 
 bool mainWindow::ensureConfigPathExists( void )
@@ -107,6 +110,7 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 	QWidget * hbox = new QWidget( this );
 	QHBoxLayout * hbox_layout = new QHBoxLayout( hbox );
 	hbox_layout->setMargin( 0 );
+	hbox_layout->setSpacing( 0 );
 
 	// create splitter, which is used for splitting sidebar-workspaces
 	// from main-workspace
@@ -116,14 +120,13 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 #endif
 
 	// create sidebar
-	m_sideBar = new italcSideBar( italcSideBar::Vertical, hbox,
-								m_splitter );
-	m_sideBar->setStyle( italcSideBar::VSNET/*KDEV3ICON*/ );
+	m_sideBar = new italcSideBar( italcSideBar::VSNET, hbox, m_splitter );
 
 
 
 	QScrollArea * sa = new QScrollArea( m_splitter );
 	sa->setBackgroundRole( QPalette::Dark );
+	sa->setFrameStyle( QFrame::NoFrame );
 	m_splitter->setStretchFactor( m_splitter->indexOf( sa ), 10 );
 	m_workspace = new clientWorkspace( sa );
 
@@ -134,7 +137,8 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 	m_classroomManager = new classroomManager( this, twp );
 	m_snapshotList = new snapshotList( this, twp );
 	m_configWidget = new configWidget( this, twp );
-	m_supportWidget = new supportWidget( this, twp );
+
+	m_workspace->m_contextMenu = m_classroomManager->quickSwitchMenu();
 
 	// append sidebar-workspaces to sidebar
 	int id = 0;
@@ -142,7 +146,6 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 	m_sideBar->appendTab( m_classroomManager, ++id );
 	m_sideBar->appendTab( m_snapshotList, ++id );
 	m_sideBar->appendTab( m_configWidget, ++id );
-	m_sideBar->appendTab( m_supportWidget, ++id );
 	m_sideBar->setPosition( italcSideBar::Left );
 	m_sideBar->setTab( m_openedTabInSideBar, TRUE );
 
@@ -155,6 +158,7 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 
 	// create the action-toolbar
 	m_toolBar = new toolBar( tr( "Actions" ), this );
+	m_toolBar->layout()->setSpacing( 4 );
 	m_toolBar->setMovable( FALSE );
 	m_toolBar->setObjectName( "maintoolbar" );
 	m_toolBar->toggleViewAction()->setEnabled( FALSE );
@@ -175,46 +179,53 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 
 	m_modeGroup = new QButtonGroup( this );
 
+	QAction * a;
+
+	a = new QAction( QIcon( ":/resources/overview_mode.png" ),
+						tr( "Overview mode" ), this );
+	m_sysTrayActions << a;
 	toolButton * overview_mode = new toolButton(
-			QPixmap( ":/resources/overview_mode.png" ),
-			tr( "Overview" ), QString::null,
-			tr( "Overview mode" ),
+			a, tr( "Overview" ), QString::null,
 			tr( "This is the default mode in iTALC and allows you "
 				"to have an overview over all visible "
 				"computers. Also click on this button for "
 				"unlocking locked workstations or for leaving "
 				"demo-mode." ),
-			NULL, NULL, m_toolBar );
+			this, SLOT( mapOverview() ), m_toolBar );
 
+
+	a = new QAction( QIcon( ":/resources/fullscreen_demo.png" ),
+						tr( "Fullscreen demo" ), this );
+	m_sysTrayActions << a;
 	toolButton * fsdemo_mode = new toolButton(
-			QPixmap( ":/resources/fullscreen_demo.png" ),
-			tr( "Demo on" ), tr( "Demo off" ),
-			tr( "Fullscreen demo" ),
+			a, tr( "Fullscreen Demo" ), tr( "Stop Demo" ),
 			tr( "In this mode your screen is being displayed on "
 				"all shown computers. Furthermore the users "
 				"aren't able to do something else as all input "
 				"devices are locked in this mode." ),
-			NULL, NULL, m_toolBar );
+			this, SLOT( mapFullscreenDemo() ), m_toolBar );
 
+	a = new QAction( QIcon( ":/resources/window_demo.png" ),
+						tr( "Window demo" ), this );
+	m_sysTrayActions << a;
 	toolButton * windemo_mode = new toolButton(
-			QPixmap( ":/resources/window_demo.png" ),
-			tr( "Demo/window on" ), tr( "Demo/window off" ),
-			tr( "Window demo" ),
+			a, tr( "Window Demo" ), tr( "Stop Demo" ),
 			tr( "In this mode your screen being displayed in a "
 				"window on all shown computers. The users are "
 				"able to switch to other windows and thus "
 				"can continue to work." ),
-			NULL, NULL, m_toolBar );
+			this, SLOT( mapWindowDemo() ), m_toolBar );
 
+	a = new QAction( QIcon( ":/resources/locked.png" ),
+					tr( "Lock/unlock desktops" ), this );
+	m_sysTrayActions << a;
 	toolButton * lock_mode = new toolButton(
-			QPixmap( ":/resources/locked.png" ),
-			tr( "Lock all" ), tr( "Unlock all" ),
-			tr( "Lock/unlock desktops" ),
+			a, tr( "Lock all" ), tr( "Unlock all" ),
 			tr( "To have all user's full attention you can lock "
 				"their desktops using this button. "
 				"In this mode all input devices are locked and "
 				"the screen is black." ),
-			NULL, NULL, m_toolBar );
+			this, SLOT( mapScreenLock() ), m_toolBar );
 
 	overview_mode->setCheckable( TRUE );
 	fsdemo_mode->setCheckable( TRUE );
@@ -227,33 +238,35 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 	m_modeGroup->addButton( lock_mode, client::Mode_Locked );
 
 	overview_mode->setChecked( TRUE );
-	connect( m_modeGroup, SIGNAL( buttonClicked( int ) ),
-			this, SLOT( changeGlobalClientMode( int ) ) );
 
 
 
+	a = new QAction( QIcon( ":/resources/text_message.png" ),
+					tr( "Send text message" ), this );
+//	m_sysTrayActions << a;
 	toolButton * text_msg = new toolButton(
-			QPixmap( ":/resources/text_message.png" ),
-			tr( "Text message" ), QString::null,
-			tr( "Send text message" ),
+			a, tr( "Text message" ), QString::null,
 			tr( "Use this button to send a text message to all "
 				"users e.g. to tell them new tasks etc." ),
 			m_classroomManager, SLOT( sendMessage() ), m_toolBar );
 
 
+	a = new QAction( QIcon( ":/resources/power_on.png" ),
+					tr( "Power on computers" ), this );
+	m_sysTrayActions << a;
 	toolButton * power_on = new toolButton(
-			QPixmap( ":/resources/power_on.png" ),
-			tr( "Power on" ), QString::null,
-			tr( "Power on computers" ),
+			a, tr( "Power on" ), QString::null,
 			tr( "Click this button to power on all visible "
 				"computers. This way you do not have to turn "
 				"on each computer by hand." ),
-			m_classroomManager, SLOT( powerOnClients() ), m_toolBar );
+			m_classroomManager, SLOT( powerOnClients() ),
+								m_toolBar );
 
+	a = new QAction( QIcon( ":/resources/power_off.png" ),
+					tr( "Power down computers" ), this );
+	m_sysTrayActions << a;
 	toolButton * power_off = new toolButton(
-			QPixmap( ":/resources/power_off.png" ),
-			tr( "Power down" ), QString::null,
-			tr( "Power down computers" ),
+			a, tr( "Power down" ), QString::null,
 			tr( "To power down all shown computers (e.g. after "
 				"the lesson has finished) you can click this "
 				"button." ),
@@ -269,6 +282,14 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 				"according user on all visible computers." ),
 			m_classroomManager, SLOT( remoteLogon() ), m_toolBar );
 
+	toolButton * directsupport = new toolButton(
+			QPixmap( ":/resources/remote_control.png" ),
+			tr( "Support" ), QString::null,
+			tr( "Direct support" ),
+			tr( "If you need to support someone at a certain "
+				"computer you can click this button and enter "
+				"the according hostname or IP afterwards." ),
+			m_classroomManager, SLOT( directSupport() ), m_toolBar );
 
 	toolButton * adjust_size = new toolButton(
 			QPixmap( ":/resources/adjust_size.png" ),
@@ -284,23 +305,41 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 			tr( "Auto view" ), QString::null,
 			tr( "Auto re-arrange windows and their size" ),
 			tr( "When clicking this button all visible windows "
-				"are re-arranged and adjusted." ),
+					"are re-arranged and adjusted." ),
 			m_classroomManager, SLOT( arrangeWindows() ), m_toolBar );
 
-	m_toolBar->addWidget( scr );
-	m_toolBar->addWidget( overview_mode );
-	m_toolBar->addWidget( fsdemo_mode );
-	m_toolBar->addWidget( windemo_mode );
-	m_toolBar->addWidget( lock_mode );
-	m_toolBar->addWidget( text_msg );
-	m_toolBar->addWidget( power_on );
-	m_toolBar->addWidget( power_off );
-	m_toolBar->addWidget( remotelogon );
-	m_toolBar->addWidget( adjust_size );
-	m_toolBar->addWidget( auto_arrange );
+	scr->addTo( m_toolBar );
+	overview_mode->addTo( m_toolBar );
+	fsdemo_mode->addTo( m_toolBar );
+	windemo_mode->addTo( m_toolBar );
+	lock_mode->addTo( m_toolBar );
+	text_msg->addTo( m_toolBar );
+	power_on->addTo( m_toolBar );
+	power_off->addTo( m_toolBar );
+	remotelogon->addTo( m_toolBar );
+	directsupport->addTo( m_toolBar );
+	adjust_size->addTo( m_toolBar );
+	auto_arrange->addTo( m_toolBar );
 
 	restoreState( QByteArray::fromBase64(
 				m_classroomManager->winCfg().toUtf8() ) );
+	QStringList hidden_buttons = m_classroomManager->toolBarCfg().
+								split( '#' );
+	foreach( QAction * a, m_toolBar->actions() )
+	{
+		if( hidden_buttons.contains( a->text() ) )
+		{
+			a->setVisible( FALSE );
+		}
+	}
+
+	foreach( KMultiTabBarTab * tab, m_sideBar->tabs() )
+	{
+		if( hidden_buttons.contains( tab->text() ) )
+		{
+			tab->setTabVisible( FALSE );
+		}
+	}
 
 	while( 1 )
 	{
@@ -323,13 +362,10 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 				"Otherwise you won't be able to access "
 						"computers using iTALC." ) );
 		}
-		else
-		{
-			break;
-		}
+		break;
 	}
 	m_localISD = new isdConnection( QHostAddress(
-				QHostAddress::LocalHost).toString() +
+				__isd_host ).toString() +
 					":" + QString::number( __isd_port ) );
 	if( m_localISD->open() != isdConnection::Connected )
 	{
@@ -343,8 +379,24 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 		return;
 	}
 
-	m_localISD->demoServerStop();
-	m_localISD->demoServerRun( __demo_quality, localSystem::freePort() );
+	m_localISD->demoServerRun( __demo_quality,
+						localSystem::freePort( 5858 ) );
+
+
+	m_localISD->hideTrayIcon();
+
+	QIcon icon( ":/resources/icon16.png" );
+	icon.addFile( ":/resources/icon22.png" );
+	icon.addFile( ":/resources/icon32.png" );
+
+	__systray_icon = new QSystemTrayIcon( icon, this );
+	__systray_icon->setToolTip( tr( "iTALC Master Control" ) );
+	__systray_icon->show();
+	connect( __systray_icon, SIGNAL( activated(
+					QSystemTrayIcon::ActivationReason ) ),
+		this, SLOT( handleSystemTrayEvent(
+					QSystemTrayIcon::ActivationReason ) ) );
+
 
 	QTimer::singleShot( 2000, m_classroomManager, SLOT( updateClients() ) );
 
@@ -357,12 +409,28 @@ mainWindow::mainWindow( int _rctrl_screen ) :
 mainWindow::~mainWindow()
 {
 	m_classroomManager->doCleanupWork();
+
+#ifdef BUILD_WIN32
+	qApp->processEvents( QEventLoop::AllEvents, 3000 );
+	localSystem::sleep( 3000 );
+#endif
+
+	// also delets clients
 	delete m_workspace;
 
+	m_localISD->gracefulClose();
 
-	m_localISD->demoServerStop();
 	delete m_localISD;
 	m_localISD = NULL;
+
+	__systray_icon->hide();
+	delete __systray_icon;
+
+#ifdef BUILD_WIN32
+	qApp->processEvents( QEventLoop::AllEvents, 3000 );
+	localSystem::sleep( 3000 );
+	exit( 0 );
+#endif
 }
 
 
@@ -393,6 +461,12 @@ void mainWindow::closeEvent( QCloseEvent * _ce )
 	delete m_updateThread;
 	m_updateThread = NULL;
 
+	QList<client *> clients = m_workspace->findChildren<client *>();
+	foreach( client * c, clients )
+	{
+		c->quit();
+	}
+
 	m_classroomManager->savePersonalConfig();
 	m_classroomManager->saveGlobalClientConfig();
 
@@ -402,7 +476,68 @@ void mainWindow::closeEvent( QCloseEvent * _ce )
 
 
 
-void mainWindow::remoteControlDisplay( const QString & _ip, bool _view_only,
+
+void mainWindow::handleSystemTrayEvent( QSystemTrayIcon::ActivationReason _r )
+{
+	switch( _r )
+	{
+		case QSystemTrayIcon::Trigger:
+			setVisible( !isVisible() );
+			break;
+		case QSystemTrayIcon::Context:
+		{
+			QMenu m( this );
+			m.addAction( __systray_icon->toolTip() )->setEnabled( FALSE );
+			foreach( QAction * a, m_sysTrayActions )
+			{
+				m.addAction( a );
+			}
+
+			m.addSeparator();
+
+			QMenu rcm( this );
+			QAction * rc = m.addAction( tr( "Remote control" ) );
+			rc->setMenu( &rcm );
+			foreach( client * c,
+					m_classroomManager->visibleClients() )
+			{
+				rcm.addAction( c->name() )->
+						setData( c->hostname() );
+			}
+			connect( &rcm, SIGNAL( triggered( QAction * ) ),
+				this,
+				SLOT( remoteControlClient( QAction * ) ) );
+
+			m.addSeparator();
+
+			QAction * qa = m.addAction(
+					QIcon( ":/resources/quit.png" ),
+					tr( "Quit" ) );
+			connect( qa, SIGNAL( triggered( bool ) ),
+					this, SLOT( close() ) );
+			m.exec( QCursor::pos() );
+			break;
+		}
+		default:
+			break;
+	}
+}
+
+
+
+
+void mainWindow::remoteControlClient( QAction * _a )
+{
+	show();
+	remoteControlDisplay( _a->data().toString(),
+				m_classroomManager->clientDblClickAction() );
+}
+
+
+
+
+void mainWindow::remoteControlDisplay( const QString & _hostname,
+						bool _view_only,
 						bool _stop_demo_afterwards )
 {
 	QWriteLocker wl( &m_rctrlLock );
@@ -410,7 +545,7 @@ void mainWindow::remoteControlDisplay( const QString & _ip, bool _view_only,
 	{
 		return;
 	}
-	m_remoteControlWidget = new remoteControlWidget( _ip, _view_only,
+	m_remoteControlWidget = new remoteControlWidget( _hostname, _view_only,
 									this );
 	int x = 0;
 	for( int i = 0; i < m_remoteControlScreen; ++i )
@@ -433,7 +568,8 @@ void mainWindow::remoteControlWidgetClosed( QObject * )
 	m_rctrlLock.unlock();
 	if( m_stopDemo )
 	{
-		m_classroomManager->changeGlobalClientMode( client::Mode_Overview );
+		m_classroomManager->changeGlobalClientMode(
+							client::Mode_Overview );
 		m_stopDemo = FALSE;
 	}
 }
@@ -510,7 +646,8 @@ void mainWindowUpdateThread::run( void )
 
 
 clientWorkspace::clientWorkspace( QScrollArea * _parent ) :
-	QWidget( _parent )
+	QWidget( _parent ),
+	m_contextMenu( NULL )
 {
 	_parent->setWidget( this );
 	_parent->setWidgetResizable( TRUE );
@@ -529,5 +666,15 @@ QSize clientWorkspace::sizeHint( void ) const
 
 
 
+
+void clientWorkspace::contextMenuEvent( QContextMenuEvent * _event )
+{
+	m_contextMenu->exec( _event->globalPos() );
+	_event->accept();
+}
+
+
+
+
 #include "main_window.moc"
 #include "italc_side_bar.moc"
