@@ -80,10 +80,10 @@ qint64 qtcpsocketDispatcher( char * _buf, const qint64 _len,
 				}
 				ret += written;
 			}
-			sock->flush();
-			//sock->waitForBytesWritten();
+			//sock->flush();
+			sock->waitForBytesWritten();
 			break;
-		case SocketGetIPBoundTo:
+		case SocketGetPeerAddress:
 			strncpy( _buf,
 		sock->peerAddress().toString().toAscii().constData(), _len );
 			break;
