@@ -1,16 +1,16 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS><TS version="1.1" language="uk_UA">
 <context>
-    <name></name>
+    <name>@default</name>
     <message>
         <location filename="../src/main.cpp" line="82"/>
         <source>File does not exist</source>
-        <translation>Файла не існує</translation>
+        <translation type="unfinished">Файла не існує</translation>
     </message>
     <message>
         <location filename="../src/main.cpp" line="86"/>
         <source>The file %1 could not be found. Please check this and try again.</source>
-        <translation>Файл %1 неможливо знайти. Будь ласка, перевірте його наявніть і спробуйте ще раз.</translation>
+        <translation type="unfinished">Файл %1 неможливо знайти. Будь ласка, перевірте його наявніть і спробуйте ще раз.</translation>
     </message>
 </context>
 <context>
@@ -255,72 +255,72 @@ Click &apos;Next&apos; to continue.</source>
 <context>
     <name>setupWizard</name>
     <message>
-        <location filename="../src/dialogs.cpp" line="153"/>
+        <location filename="../src/dialogs.cpp" line="155"/>
         <source>Confirm overwrite</source>
         <translation>Підтвердження перезапису</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="154"/>
+        <location filename="../src/dialogs.cpp" line="156"/>
         <source>Do you want to overwrite %1?</source>
         <translation>Ви бажаєте перезаписати %1?</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="211"/>
+        <location filename="../src/dialogs.cpp" line="212"/>
         <source>Installing iTALC</source>
         <translation>Встановлення iTALC</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="213"/>
+        <location filename="../src/dialogs.cpp" line="214"/>
         <source>Copying files...</source>
         <translation>Копіювання файлів...</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="278"/>
+        <location filename="../src/dialogs.cpp" line="279"/>
         <source>Registering ICA as service...</source>
         <translation>Реєстрація служби ICA...</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="321"/>
+        <location filename="../src/dialogs.cpp" line="322"/>
         <source>Creating/importing keys...</source>
         <translation>Створення/імпорт ключів...</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="353"/>
+        <location filename="../src/dialogs.cpp" line="354"/>
         <source>Creating shortcuts...</source>
         <translation>Створення посилань...</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="375"/>
+        <location filename="../src/dialogs.cpp" line="376"/>
         <source>Could not create directory</source>
         <translation>Не вдалося створити теку</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="379"/>
+        <location filename="../src/dialogs.cpp" line="380"/>
         <source>Could not create directory %1! Make sure you have the neccessary rights and try again!</source>
         <translation>Не вдалося створити теку %1! Переконайтеся, що Ви маєте на це дозвіл і спробуйте ще раз!</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="392"/>
+        <location filename="../src/dialogs.cpp" line="393"/>
         <source>Cancel setup</source>
         <translation>Скасувати встановлення</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="394"/>
+        <location filename="../src/dialogs.cpp" line="395"/>
         <source>Are you sure want to quit setup? iTALC is not installed completely yet!</source>
         <translation>Ви справді бажаєте вийти з програми встановлення? iTALC ще не встановлено повністю!</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="414"/>
+        <location filename="../src/dialogs.cpp" line="416"/>
         <source>Next</source>
         <translation>Далі</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="473"/>
+        <location filename="../src/dialogs.cpp" line="475"/>
         <source>Finish</source>
         <translation>Завершити</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="479"/>
+        <location filename="../src/dialogs.cpp" line="481"/>
         <source>Quit</source>
         <translation>Вийти</translation>
     </message>
@@ -328,17 +328,17 @@ Click &apos;Next&apos; to continue.</source>
 <context>
     <name>setupWizardPageInstallDir</name>
     <message>
-        <location filename="../src/dialogs.cpp" line="568"/>
+        <location filename="../src/dialogs.cpp" line="590"/>
         <source>Directory does not exist</source>
         <translation>Теки не існує</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="570"/>
+        <location filename="../src/dialogs.cpp" line="592"/>
         <source>The specified directory does not exist. Do you want to create it?</source>
         <translation>Заданої теки не існує. Ви бажаєте її створити?</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="592"/>
+        <location filename="../src/dialogs.cpp" line="614"/>
         <source>Choose installation directory</source>
         <translation>Вибір теки встановлення</translation>
     </message>
@@ -346,17 +346,17 @@ Click &apos;Next&apos; to continue.</source>
 <context>
     <name>setupWizardPageKeyDirs</name>
     <message>
-        <location filename="../src/dialogs.cpp" line="826"/>
+        <location filename="../src/dialogs.cpp" line="848"/>
         <source>Choose public key directory</source>
         <translation>Вибір теки публічного ключа</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="840"/>
+        <location filename="../src/dialogs.cpp" line="862"/>
         <source>Choose private key directory</source>
         <translation>Вибір теки особистого ключа</translation>
     </message>
     <message>
-        <location filename="../src/dialogs.cpp" line="854"/>
+        <location filename="../src/dialogs.cpp" line="876"/>
         <source>Choose public key export directory</source>
         <translation>Вибір теки експорту публічного ключа</translation>
     </message>
@@ -364,7 +364,7 @@ Click &apos;Next&apos; to continue.</source>
 <context>
     <name>setupWizardPageSecurityOptions</name>
     <message>
-        <location filename="../src/dialogs.cpp" line="728"/>
+        <location filename="../src/dialogs.cpp" line="750"/>
         <source>Choose directory for key-import</source>
         <translation>Вибір теки імпорту ключа</translation>
     </message>
