@@ -28,6 +28,7 @@
 #endif
 
 #include <QtCore/QProcess>
+#include <QtGui/QCloseEvent>
 #include <QtGui/QCompleter>
 #include <QtGui/QDirModel>
 #include <QtGui/QFileDialog>
@@ -67,7 +68,8 @@ setupWizard::setupWizard() :
 	m_installDocs( FALSE ),
 	m_saveInstallSettings( FALSE ),
 	m_widgetStack(),
-	m_idx( 0 )
+	m_idx( 0 ),
+	m_closeOk( FALSE )
 {
 	setupUi( this );
 	DEFAULT_INSTALL_DIR =
@@ -192,8 +194,7 @@ void setupWizard::doInstallation( bool _quiet )
 		"QtNetwork4.dll"	<<
 		"QtXml4.dll"		<<
 		"vnchooks.dll"		<<
-		"italc_core.dll"	<<
-		"userinfo.exe"
+		"italc_core.dll"
 #else
 		"libssl.so.0.9.8"	<<
 		"libcrypto.so.0.9.8"	<<
@@ -278,8 +279,8 @@ void setupWizard::doInstallation( bool _quiet )
 	pd.setLabelText( tr( "Registering ICA as service..." ) );
 	qApp->processEvents();
 
-	QProcess::execute( d + "ica", QStringList() << quiet_opt <<
-							"-registerservice"  );
+	QProcess::execute( d + "ica", QStringList( quiet_opt ) <<
+							"-registerservice" );
 	if( did_overwrite_ica )
 	{
 		qApp->processEvents();
@@ -397,6 +398,7 @@ void setupWizard::reject( void )
 			==
 						QMessageBox::Yes )
 	{
+		m_closeOk = TRUE;
 		QDialog::reject();
 	}
 }
@@ -484,6 +486,26 @@ void setupWizard::next( void )
 
 
 
+void setupWizard::closeEvent( QCloseEvent * _ev )
+{
+	if( m_closeOk )
+	{
+		QDialog::closeEvent( _ev );
+	}
+	else
+	{
+		reject();
+		if( !m_closeOk )
+		{
+			_ev->ignore();
+		}
+	}
+}
+
+
+
+
+
 setupWizardPage::setupWizardPage( setupWizard * _wiz ) :
 	QWidget( _wiz->contentWidget ),
 	m_setupWizard( _wiz )
